@@ -6,6 +6,7 @@
 
     assert.commandWorked(st.s0.adminCommand({enablesharding: "test"}));
     st.ensurePrimaryShard('test', st.shard1.shardName);
+
     assert.commandWorked(st.s0.adminCommand({shardcollection: "test.foo", key: {num: 1}}));
 
     assert.writeOK(st.s0.getDB('test').existing.insert({_id: 1}));
@@ -21,9 +22,7 @@
         to: st.getOther(st.getPrimaryShard("test")).name
     }));
 
-    assert.commandWorked(st.s0.adminCommand({flushRouterConfig: 1}));
-
-    assert.eq(1, st.s0.getDB('test').existing.count({_id: 1}));  // SERVER-2828
+    assert.eq(1, st.s0.getDB('test').existing.count({_id: 1}));
     assert.eq(1, st.s1.getDB('test').existing.count({_id: 1}));
 
     st.stop();
