@@ -1,30 +1,33 @@
 // bsonelement.h
 
-/*    Copyright 2009 10gen Inc.
+
+/**
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects
- *    for all of the code used other than as permitted herein. If you modify
- *    file(s) with this exception, you may extend this exception to your
- *    version of the file(s), but you are not obligated to do so. If you do not
- *    wish to do so, delete this exception statement from your version. If you
- *    delete this exception statement from all source files in the program,
- *    then also delete it in the license file.
+ *    must comply with the Server Side Public License in all respects for
+ *    all of the code used other than as permitted herein. If you modify file(s)
+ *    with this exception, you may extend this exception to your version of the
+ *    file(s), but you are not obligated to do so. If you do not wish to do so,
+ *    delete this exception statement from your version. If you delete this
+ *    exception statement from all source files in the program, then also delete
+ *    it in the license file.
  */
 
 #pragma once
@@ -35,8 +38,11 @@
 #include <string>
 #include <vector>
 
+#include "mongo/base/data_range.h"
 #include "mongo/base/data_type_endian.h"
 #include "mongo/base/data_view.h"
+#include "mongo/base/string_data_comparator_interface.h"
+#include "mongo/bson/bson_comparator_interface_base.h"
 #include "mongo/bson/bsontypes.h"
 #include "mongo/bson/oid.h"
 #include "mongo/bson/timestamp.h"
@@ -54,9 +60,6 @@ typedef BSONElement be;
 typedef BSONObj bo;
 typedef BSONObjBuilder bob;
 
-/* l and r MUST have same type when called: check that first. */
-int compareElementValues(const BSONElement& l, const BSONElement& r);
-
 /** BSONElement represents an "element" in a BSONObj.  So for the object { a : 3, b : "abc" },
     'a : 3' is the first element (key+value).
 
@@ -72,7 +75,35 @@ int compareElementValues(const BSONElement& l, const BSONElement& r);
 */
 class BSONElement {
 public:
-    /** These functions, which start with a capital letter, throw a MsgAssertionException if the
+    // Declared in bsonobj_comparator_interface.h.
+    class ComparatorInterface;
+
+    /**
+     * Operator overloads for relops return a DeferredComparison which can subsequently be evaluated
+     * by a BSONObj::ComparatorInterface.
+     */
+    using DeferredComparison = BSONComparatorInterfaceBase<BSONElement>::DeferredComparison;
+
+    /**
+     * Set of rules that dictate the behavior of the comparison APIs.
+     */
+    using ComparisonRules = BSONComparatorInterfaceBase<BSONElement>::ComparisonRules;
+    using ComparisonRulesSet = BSONComparatorInterfaceBase<BSONElement>::ComparisonRulesSet;
+
+    /**
+     * Compares two BSON elements of the same canonical type.
+     *
+     * Returns <0 if 'l' is less than the element 'r'.
+     *         >0 if 'l' is greater than the element 'r'.
+     *          0 if 'l' is equal to the element 'r'.
+     */
+    static int compareElements(const BSONElement& l,
+                               const BSONElement& r,
+                               ComparisonRulesSet rules,
+                               const StringData::ComparatorInterface* comparator);
+
+
+    /** These functions, which start with a capital letter, throw if the
         element is not of the required type. Example:
 
         std::string foo = obj["foo"].String(); // std::exception if not a std::string type or DNE
@@ -87,7 +118,12 @@ public:
         return chk(mongo::Date).date();
     }
     double Number() const {
-        return chk(isNumber()).number();
+        uassert(13118,
+                str::stream() << "expected " << fieldName()
+                              << " to have a numeric type, but it is a "
+                              << type(),
+                isNumber());
+        return number();
     }
     Decimal128 Decimal() const {
         return chk(NumberDecimal)._numberDecimal();
@@ -108,18 +144,12 @@ public:
     mongo::OID OID() const {
         return chk(jstOID).__oid();
     }
-    void Null() const {
-        chk(isNull());
-    }  // throw MsgAssertionException if not null
-    void OK() const {
-        chk(ok());
-    }  // throw MsgAssertionException if element DNE
 
     /** @return the embedded object associated with this field.
         Note the returned object is a reference to within the parent bson object. If that
         object is out of scope, this pointer will no longer be valid. Call getOwned() on the
         returned BSONObj if you need your own copy.
-        throws UserException if the element is not of type object.
+        throws AssertionException if the element is not of type object.
     */
     BSONObj Obj() const;
 
@@ -178,6 +208,7 @@ public:
     void toString(StringBuilder& s,
                   bool includeFieldName = true,
                   bool full = false,
+                  bool redactValues = false,
                   int depth = 0) const;
     std::string jsonString(JsonStringFormat format,
                            bool includeFieldNames = true,
@@ -195,7 +226,7 @@ public:
     /** retrieve a field within this element
         throws exception if *this is not an embedded object
     */
-    BSONElement operator[](const std::string& field) const;
+    BSONElement operator[](StringData field) const;
 
     /** See canonicalizeBSONType in bsontypes.h */
     int canonicalType() const {
@@ -209,11 +240,12 @@ public:
         return type() == EOO;
     }
 
-    /** Size of the element.
-        @param maxLen If maxLen is specified, don't scan more than maxLen bytes to calculate size.
-    */
-    int size(int maxLen) const;
-    int size() const;
+    /**
+     * Size of the element.
+     */
+    int size() const {
+        return totalSize;
+    }
 
     /** Wrap this element up as a singleton object. */
     BSONObj wrap() const;
@@ -235,8 +267,6 @@ public:
      * NOTE: size includes the NULL terminator.
      */
     int fieldNameSize() const {
-        if (fieldNameSize_ == -1)
-            fieldNameSize_ = (int)strlen(fieldName()) + 1;
         return fieldNameSize_;
     }
 
@@ -281,9 +311,6 @@ public:
     */
     bool trueValue() const;
 
-    /** True if number, string, bool, date, OID */
-    bool isSimpleType() const;
-
     /** True if element is of a numeric type. */
     bool isNumber() const;
 
@@ -299,7 +326,9 @@ public:
 
     /** Return decimal128 value for this field. MUST be NumberDecimal type. */
     Decimal128 _numberDecimal() const {
-        return Decimal128(ConstDataView(value()).read<LittleEndian<Decimal128::Value>>());
+        uint64_t low = ConstDataView(value()).read<LittleEndian<long long>>();
+        uint64_t high = ConstDataView(value() + sizeof(long long)).read<LittleEndian<long long>>();
+        return Decimal128(Decimal128::Value({low, high}));
     }
 
     /** Return long long value for this field. MUST be NumberLong type. */
@@ -321,6 +350,11 @@ public:
      *  very small doubles -> LLONG_MIN  */
     long long safeNumberLong() const;
 
+    /** This safeNumberLongForHash() function does the same thing as safeNumberLong, but it
+     *  preserves edge-case behavior from older versions.
+     */
+    long long safeNumberLongForHash() const;
+
     /** Retrieve decimal value for the element safely. */
     Decimal128 numberDecimal() const;
 
@@ -453,6 +487,19 @@ public:
         return (BinDataType)c;
     }
 
+    std::vector<uint8_t> _binDataVector() const {
+        if (binDataType() != ByteArrayDeprecated) {
+            return std::vector<uint8_t>(reinterpret_cast<const uint8_t*>(value()) + 5,
+                                        reinterpret_cast<const uint8_t*>(value()) + 5 +
+                                            valuestrsize());
+        } else {
+            // Skip the extra int32 size
+            return std::vector<uint8_t>(reinterpret_cast<const uint8_t*>(value()) + 4,
+                                        reinterpret_cast<const uint8_t*>(value()) + 4 +
+                                            valuestrsize() - 4);
+        }
+    }
+
     /** Retrieve the regex std::string for a Regex element */
     const char* regex() const {
         verify(type() == RegEx);
@@ -465,45 +512,71 @@ public:
         return p + strlen(p) + 1;
     }
 
-    /** like operator== but doesn't check the fieldname,
-        just the value.
-    */
-    bool valuesEqual(const BSONElement& r) const {
-        return woCompare(r, false) == 0;
-    }
-
-    /** Returns true if elements are equal. */
-    bool operator==(const BSONElement& r) const {
-        return woCompare(r, true) == 0;
-    }
-    /** Returns true if elements are unequal. */
-    bool operator!=(const BSONElement& r) const {
-        return !operator==(r);
-    }
-
-    /** Well ordered comparison.
-        @return <0: l<r. 0:l==r. >0:l>r
-        order by type, field name, and field value.
-        If considerFieldName is true, pay attention to the field name.
-    */
-    int woCompare(const BSONElement& e, bool considerFieldName = true) const;
+    //
+    // Comparison API.
+    //
+    // BSONElement instances can be compared via a raw bytewise comparison or a logical comparison.
+    //
+    // Logical comparison can be done either using woCompare() or with operator overloads. Most
+    // callers should prefer operator overloads. Note that the operator overloads return a
+    // DeferredComparison, which must subsequently be evaluated by a
+    // BSONElement::ComparatorInterface. See bsonelement_comparator_interface.h for details.
+    //
 
     /**
-     * Functor compatible with std::hash for std::unordered_{map,set}
-     * Warning: The hash function is subject to change. Do not use in cases where hashes need
-     *          to be consistent across versions.
+     * Compares the raw bytes of the two BSONElements, including the field names. This will treat
+     * different types (e.g. integers and doubles) as distinct values, even if they have the same
+     * field name and bit pattern in the value portion of the BSON element.
      */
-    struct Hasher {
-        size_t operator()(const BSONElement& elem) const;
-    };
+    bool binaryEqual(const BSONElement& rhs) const;
+
+    /**
+     * Compares the raw bytes of the two BSONElements, excluding the field names. This will treat
+     * different types (e.g integers and doubles) as distinct values, even if they have the same bit
+     * pattern in the value portion of the BSON element.
+     */
+    bool binaryEqualValues(const BSONElement& rhs) const;
+
+    /**
+     * Compares two BSON Elements using the rules specified by 'rules' and the 'comparator' for
+     * string comparisons.
+     *
+     * Returns <0 if 'this' is less than 'elem'.
+     *         >0 if 'this' is greater than 'elem'.
+     *          0 if 'this' is equal to 'elem'.
+     */
+    int woCompare(const BSONElement& elem,
+                  ComparisonRulesSet rules = ComparisonRules::kConsiderFieldName,
+                  const StringData::ComparatorInterface* comparator = nullptr) const;
+
+    DeferredComparison operator<(const BSONElement& other) const {
+        return DeferredComparison(DeferredComparison::Type::kLT, *this, other);
+    }
+
+    DeferredComparison operator<=(const BSONElement& other) const {
+        return DeferredComparison(DeferredComparison::Type::kLTE, *this, other);
+    }
+
+    DeferredComparison operator>(const BSONElement& other) const {
+        return DeferredComparison(DeferredComparison::Type::kGT, *this, other);
+    }
+
+    DeferredComparison operator>=(const BSONElement& other) const {
+        return DeferredComparison(DeferredComparison::Type::kGTE, *this, other);
+    }
+
+    DeferredComparison operator==(const BSONElement& other) const {
+        return DeferredComparison(DeferredComparison::Type::kEQ, *this, other);
+    }
+
+    DeferredComparison operator!=(const BSONElement& other) const {
+        return DeferredComparison(DeferredComparison::Type::kNE, *this, other);
+    }
 
     const char* rawdata() const {
         return data;
     }
 
-    /** 0 == Equality, just not defined yet */
-    int getGtLtOp(int def = 0) const;
-
     /** Constructs an empty element */
     BSONElement();
 
@@ -537,6 +610,31 @@ public:
         return Timestamp();
     }
 
+    const std::array<unsigned char, 16> uuid() const {
+        int len = 0;
+        const char* data = nullptr;
+        if (type() == BinData && binDataType() == BinDataType::newUUID)
+            data = binData(len);
+        uassert(ErrorCodes::InvalidUUID,
+                "uuid must be a 16-byte binary field with UUID (4) subtype",
+                len == 16);
+        std::array<unsigned char, 16> result;
+        memcpy(&result, data, len);
+        return result;
+    }
+
+    const std::array<unsigned char, 16> md5() const {
+        int len = 0;
+        const char* data = nullptr;
+        if (type() == BinData && binDataType() == BinDataType::MD5Type)
+            data = binData(len);
+        uassert(40437, "md5 must be a 16-byte binary field with MD5 (5) subtype", len == 16);
+        std::array<unsigned char, 16> result;
+        memcpy(&result, data, len);
+        return result;
+    }
+
+
     Date_t timestampTime() const {
         unsigned long long t = ConstDataView(value() + 4).read<LittleEndian<unsigned int>>();
         return Date_t::fromMillisSinceEpoch(t * 1000);
@@ -561,82 +659,79 @@ public:
         return mongo::OID::from(start);
     }
 
-    /** this does not use fieldName in the comparison, just the value */
-    bool operator<(const BSONElement& other) const {
-        int x = (int)canonicalType() - (int)other.canonicalType();
-        if (x < 0)
-            return true;
-        else if (x > 0)
-            return false;
-        return compareElementValues(*this, other) < 0;
+    // @param maxLen don't scan more than maxLen bytes
+    explicit BSONElement(const char* d) : data(d) {
+        if (eoo()) {
+            fieldNameSize_ = 0;
+            totalSize = 1;
+        } else {
+            fieldNameSize_ = strlen(d + 1 /*skip type*/) + 1 /*include NUL byte*/;
+            totalSize = computeSize();
+        }
     }
 
-    // @param maxLen don't scan more than maxLen bytes
-    explicit BSONElement(const char* d, int maxLen) : data(d) {
+    struct CachedSizeTag {};  // Opts in to next constructor.
+
+    /**
+     * Construct a BSONElement where you already know the length of the name and/or the total size
+     * of the element. fieldNameSize includes the null terminator. You may pass -1 for either or
+     * both sizes to indicate that they are unknown and should be computed.
+     */
+    BSONElement(const char* d, int fieldNameSize, int totalSize, CachedSizeTag) : data(d) {
         if (eoo()) {
-            totalSize = 1;
             fieldNameSize_ = 0;
+            this->totalSize = 1;
         } else {
-            totalSize = -1;
-            fieldNameSize_ = -1;
-            if (maxLen != -1) {
-                size_t size = strnlen(fieldName(), maxLen - 1);
-                uassert(10333, "Invalid field name", size < size_t(maxLen - 1));
-                fieldNameSize_ = size + 1;
+            if (fieldNameSize == -1) {
+                fieldNameSize_ = strlen(d + 1 /*skip type*/) + 1 /*include NUL byte*/;
+            } else {
+                fieldNameSize_ = fieldNameSize;
+            }
+            if (totalSize == -1) {
+                this->totalSize = computeSize();
+            } else {
+                this->totalSize = totalSize;
             }
         }
     }
 
-    explicit BSONElement(const char* d) : data(d) {
-        fieldNameSize_ = -1;
-        totalSize = -1;
-        if (eoo()) {
-            fieldNameSize_ = 0;
-            totalSize = 1;
-        }
-    }
-
-    struct FieldNameSizeTag {};  // For disambiguation with ctor taking 'maxLen' above.
-
-    /** Construct a BSONElement where you already know the length of the name. The value
-     *  passed here includes the null terminator. The data pointed to by 'd' must not
-     *  represent an EOO. You may pass -1 to indicate that you don't actually know the
-     *  size.
-     */
-    BSONElement(const char* d, int fieldNameSize, FieldNameSizeTag)
-        : data(d),
-          fieldNameSize_(fieldNameSize)  // internal size includes null terminator
-          ,
-          totalSize(-1) {}
-
     std::string _asCode() const;
 
     template <typename T>
     bool coerce(T* out) const;
 
+    /**
+     * Constant double representation of 2^63, the smallest value that will overflow a long long.
+     *
+     * It is not safe to obtain this value by casting std::numeric_limits<long long>::max() to
+     * double, because the conversion loses precision, and the C++ standard leaves it up to the
+     * implementation to decide whether to round up to 2^63 or round down to the next representable
+     * value (2^63 - 2^10).
+     */
+    static const double kLongLongMaxPlusOneAsDouble;
+
 private:
     const char* data;
-    mutable int fieldNameSize_;  // cached value
-
-    mutable int totalSize; /* caches the computed size */
+    int fieldNameSize_;  // internal size includes null terminator
+    int totalSize;
 
     friend class BSONObjIterator;
+    friend class BSONObjStlIterator;
     friend class BSONObj;
-    const BSONElement& chk(int t) const {
+    const BSONElement& chk(BSONType t) const {
         if (t != type()) {
             StringBuilder ss;
             if (eoo())
                 ss << "field not found, expected type " << t;
             else
                 ss << "wrong type for field (" << fieldName() << ") " << type() << " != " << t;
-            msgasserted(13111, ss.str());
+            uasserted(13111, ss.str());
         }
         return *this;
     }
-    const BSONElement& chk(bool expr) const {
-        massert(13118, "unexpected or missing type value in BSON object", expr);
-        return *this;
-    }
+
+    // Only called from constructors.
+    int computeSize() const;
 };
 
 inline bool BSONElement::trueValue() const {
@@ -666,9 +761,7 @@ inline bool BSONElement::isNumber() const {
     switch (type()) {
         case NumberLong:
         case NumberDouble:
-#ifdef MONGO_CONFIG_EXPERIMENTAL_DECIMAL_SUPPORT
         case NumberDecimal:
-#endif
         case NumberInt:
             return true;
         default:
@@ -676,22 +769,6 @@ inline bool BSONElement::isNumber() const {
     }
 }
 
-inline bool BSONElement::isSimpleType() const {
-    switch (type()) {
-        case NumberLong:
-        case NumberDouble:
-        case NumberInt:
-        case NumberDecimal:
-        case mongo::String:
-        case mongo::Bool:
-        case mongo::Date:
-        case jstOID:
-            return true;
-        default:
-            return false;
-    }
-}
-
 inline Decimal128 BSONElement::numberDecimal() const {
     switch (type()) {
         case NumberDouble:
@@ -699,11 +776,11 @@ inline Decimal128 BSONElement::numberDecimal() const {
         case NumberInt:
             return Decimal128(_numberInt());
         case NumberLong:
-            return Decimal128(_numberLong());
+            return Decimal128(static_cast<int64_t>(_numberLong()));
         case NumberDecimal:
             return _numberDecimal();
         default:
-            return 0;
+            return Decimal128::kNormalizedZero;
     }
 }
 
@@ -767,7 +844,7 @@ inline long long BSONElement::safeNumberLong() const {
             if (std::isnan(d)) {
                 return 0;
             }
-            if (d > (double)std::numeric_limits<long long>::max()) {
+            if (!(d < kLongLongMaxPlusOneAsDouble)) {
                 return std::numeric_limits<long long>::max();
             }
             if (d < std::numeric_limits<long long>::min()) {
@@ -780,11 +857,11 @@ inline long long BSONElement::safeNumberLong() const {
             if (d.isNaN()) {
                 return 0;
             }
-            if (d.isGreater(Decimal128(std::numeric_limits<long long>::max()))) {
-                return std::numeric_limits<long long>::max();
+            if (d.isGreater(Decimal128(std::numeric_limits<int64_t>::max()))) {
+                return static_cast<long long>(std::numeric_limits<int64_t>::max());
             }
-            if (d.isLess(Decimal128(std::numeric_limits<long long>::min()))) {
-                return std::numeric_limits<long long>::min();
+            if (d.isLess(Decimal128(std::numeric_limits<int64_t>::min()))) {
+                return static_cast<long long>(std::numeric_limits<int64_t>::min());
             }
             return numberLong();
         }
@@ -793,6 +870,38 @@ inline long long BSONElement::safeNumberLong() const {
     }
 }
 
+/**
+ * This safeNumberLongForHash() function does the same thing as safeNumberLong, but it preserves
+ * edge-case behavior from older versions. It's provided for use by hash functions that need to
+ * maintain compatibility with older versions. Don't make any changes to safeNumberLong() without
+ * ensuring that this function (which is implemented in terms of safeNumberLong()) has exactly the
+ * same behavior.
+ *
+ * Historically, safeNumberLong() used a check that would consider 2^63 to be safe to cast to
+ * int64_t, but that value actually overflows. That overflow is preserved here.
+ *
+ * The new safeNumberLong() function uses a tight bound, allowing it to correctly clamp double 2^63
+ * to the max 64-bit int (2^63 - 1).
+ */
+inline long long BSONElement::safeNumberLongForHash() const {
+    if (NumberDouble == type()) {
+        double d = numberDouble();
+        if (std::isnan(d)) {
+            return 0;
+        }
+        if (d > (double)std::numeric_limits<long long>::max()) {
+            return std::numeric_limits<long long>::max();
+        }
+        if (d < std::numeric_limits<long long>::min()) {
+            return std::numeric_limits<long long>::min();
+        }
+        return (long long)d;
+    }
+
+    // safeNumberLong() and safeNumberLongForHash() have identical behavior for non-long value.
+    return safeNumberLong();
+}
+
 inline BSONElement::BSONElement() {
     // This needs to be 2 elements because we check the strlen of data + 1 and GCC sees that as
     // accessing beyond the end of a constant string, even though we always check whether the
@@ -802,7 +911,4 @@ inline BSONElement::BSONElement() {
     fieldNameSize_ = 0;
     totalSize = 1;
 }
-
-// TODO(SERVER-14596): move to a better place; take a StringData.
-std::string escape(const std::string& s, bool escape_slash = false);
 }
