@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2009-2015 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
  *    This program is free software: you can redistribute it and/or modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -39,9 +41,16 @@
 #include "mongo/unittest/unittest.h"
 
 #include <boost/filesystem/operations.hpp>
+#include <boost/optional.hpp>
+#include <boost/optional/optional_io.hpp>
 
 namespace mongo {
 namespace {
+using transport::ConnectSSLMode;
+constexpr auto kEnableSSL = ConnectSSLMode::kEnableSSL;
+constexpr auto kDisableSSL = ConnectSSLMode::kDisableSSL;
+constexpr auto kGlobalSSLMode = ConnectSSLMode::kGlobalSSLMode;
+
 struct URITestCase {
     std::string URI;
     std::string uname;
@@ -51,10 +60,16 @@ struct URITestCase {
     size_t numservers;
     size_t numOptions;
     std::string database;
+    ConnectSSLMode sslMode;
 };
 
 struct InvalidURITestCase {
     std::string URI;
+    boost::optional<ErrorCodes::Error> code;
+    InvalidURITestCase(std::string aURI, boost::optional<ErrorCodes::Error> aCode = boost::none) {
+        URI = std::move(aURI);
+        code = std::move(aCode);
+    }
 };
 
 const ConnectionString::ConnectionType kMaster = ConnectionString::MASTER;
@@ -62,23 +77,39 @@ const ConnectionString::ConnectionType kSet = ConnectionString::SET;
 
 const URITestCase validCases[] = {
 
-    {"mongodb://user:pwd@127.0.0.1", "user", "pwd", kMaster, "", 1, 0, ""},
+    {"mongodb://user:pwd@127.0.0.1", "user", "pwd", kMaster, "", 1, 0, "", kGlobalSSLMode},
 
-    {"mongodb://user@127.0.0.1", "user", "", kMaster, "", 1, 0, ""},
+    {"mongodb://user@127.0.0.1", "user", "", kMaster, "", 1, 0, "", kGlobalSSLMode},
 
-    {"mongodb://localhost/?foo=bar", "", "", kMaster, "", 1, 1, ""},
+    {"mongodb://localhost/?foo=bar", "", "", kMaster, "", 1, 1, "", kGlobalSSLMode},
 
-    {"mongodb://localhost,/?foo=bar", "", "", kMaster, "", 1, 1, ""},
+    {"mongodb://localhost,/?foo=bar", "", "", kMaster, "", 1, 1, "", kGlobalSSLMode},
 
-    {"mongodb://user:pwd@127.0.0.1:1234", "user", "pwd", kMaster, "", 1, 0, ""},
+    {"mongodb://user:pwd@127.0.0.1:1234", "user", "pwd", kMaster, "", 1, 0, "", kGlobalSSLMode},
 
-    {"mongodb://user@127.0.0.1:1234", "user", "", kMaster, "", 1, 0, ""},
+    {"mongodb://user@127.0.0.1:1234", "user", "", kMaster, "", 1, 0, "", kGlobalSSLMode},
 
-    {"mongodb://127.0.0.1:1234/dbName?foo=a&c=b", "", "", kMaster, "", 1, 2, "dbName"},
+    {"mongodb://127.0.0.1:1234/dbName?foo=a&c=b",
+     "",
+     "",
+     kMaster,
+     "",
+     1,
+     2,
+     "dbName",
+     kGlobalSSLMode},
 
-    {"mongodb://127.0.0.1/dbName?foo=a&c=b", "", "", kMaster, "", 1, 2, "dbName"},
+    {"mongodb://127.0.0.1/dbName?foo=a&c=b", "", "", kMaster, "", 1, 2, "dbName", kGlobalSSLMode},
 
-    {"mongodb://user:pwd@127.0.0.1,/dbName?foo=a&c=b", "user", "pwd", kMaster, "", 1, 2, "dbName"},
+    {"mongodb://user:pwd@127.0.0.1,/dbName?foo=a&c=b",
+     "user",
+     "pwd",
+     kMaster,
+     "",
+     1,
+     2,
+     "dbName",
+     kGlobalSSLMode},
 
     {"mongodb://user:pwd@127.0.0.1,127.0.0.2/dbname?a=b&replicaSet=replName",
      "user",
@@ -87,7 +118,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      2,
-     "dbname"},
+     "dbname",
+     kGlobalSSLMode},
 
     {"mongodb://needs%20encoding%25%23!%3C%3E:pwd@127.0.0.1,127.0.0.2/"
      "dbname?a=b&replicaSet=replName",
@@ -97,7 +129,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      2,
-     "dbname"},
+     "dbname",
+     kGlobalSSLMode},
 
     {"mongodb://needs%20encoding%25%23!%3C%3E:pwd@127.0.0.1,127.0.0.2/"
      "db@name?a=b&replicaSet=replName",
@@ -107,7 +140,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      2,
-     "db@name"},
+     "db@name",
+     kGlobalSSLMode},
 
     {"mongodb://user:needs%20encoding%25%23!%3C%3E@127.0.0.1,127.0.0.2/"
      "dbname?a=b&replicaSet=replName",
@@ -117,7 +151,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      2,
-     "dbname"},
+     "dbname",
+     kGlobalSSLMode},
 
     {"mongodb://user:pwd@127.0.0.1,127.0.0.2/dbname?a=b&replicaSet=needs%20encoding%25%23!%3C%3E",
      "user",
@@ -126,7 +161,8 @@ const URITestCase validCases[] = {
      "needs encoding%#!<>",
      2,
      2,
-     "dbname"},
+     "dbname",
+     kGlobalSSLMode},
 
     {"mongodb://user:pwd@127.0.0.1,127.0.0.2/needsencoding%40hello?a=b&replicaSet=replName",
      "user",
@@ -135,7 +171,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      2,
-     "needsencoding@hello"},
+     "needsencoding@hello",
+     kGlobalSSLMode},
 
     {"mongodb://user:pwd@127.0.0.1,127.0.0.2/?replicaSet=replName",
      "user",
@@ -144,7 +181,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      1,
-     ""},
+     "",
+     kGlobalSSLMode},
 
     {"mongodb://user@127.0.0.1,127.0.0.2/?replicaSet=replName",
      "user",
@@ -153,7 +191,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      1,
-     ""},
+     "",
+     kGlobalSSLMode},
 
     {"mongodb://127.0.0.1,127.0.0.2/dbName?foo=a&c=b&replicaSet=replName",
      "",
@@ -162,7 +201,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      3,
-     "dbName"},
+     "dbName",
+     kGlobalSSLMode},
 
     {"mongodb://user:pwd@127.0.0.1:1234,127.0.0.2:1234/?replicaSet=replName",
      "user",
@@ -171,7 +211,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      1,
-     ""},
+     "",
+     kGlobalSSLMode},
 
     {"mongodb://user@127.0.0.1:1234,127.0.0.2:1234/?replicaSet=replName",
      "user",
@@ -180,7 +221,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      1,
-     ""},
+     "",
+     kGlobalSSLMode},
 
     {"mongodb://127.0.0.1:1234,127.0.0.1:1234/dbName?foo=a&c=b&replicaSet=replName",
      "",
@@ -189,19 +231,20 @@ const URITestCase validCases[] = {
      "replName",
      2,
      3,
-     "dbName"},
+     "dbName",
+     kGlobalSSLMode},
 
-    {"mongodb://user:pwd@[::1]", "user", "pwd", kMaster, "", 1, 0, ""},
+    {"mongodb://user:pwd@[::1]", "user", "pwd", kMaster, "", 1, 0, "", kGlobalSSLMode},
 
-    {"mongodb://user@[::1]", "user", "", kMaster, "", 1, 0, ""},
+    {"mongodb://user@[::1]", "user", "", kMaster, "", 1, 0, "", kGlobalSSLMode},
 
-    {"mongodb://[::1]/dbName?foo=a&c=b", "", "", kMaster, "", 1, 2, "dbName"},
+    {"mongodb://[::1]/dbName?foo=a&c=b", "", "", kMaster, "", 1, 2, "dbName", kGlobalSSLMode},
 
-    {"mongodb://user:pwd@[::1]:1234", "user", "pwd", kMaster, "", 1, 0, ""},
+    {"mongodb://user:pwd@[::1]:1234", "user", "pwd", kMaster, "", 1, 0, "", kGlobalSSLMode},
 
-    {"mongodb://user@[::1]:1234", "user", "", kMaster, "", 1, 0, ""},
+    {"mongodb://user@[::1]:1234", "user", "", kMaster, "", 1, 0, "", kGlobalSSLMode},
 
-    {"mongodb://[::1]:1234/dbName?foo=a&c=b", "", "", kMaster, "", 1, 2, "dbName"},
+    {"mongodb://[::1]:1234/dbName?foo=a&c=b", "", "", kMaster, "", 1, 2, "dbName", kGlobalSSLMode},
 
     {"mongodb://user:pwd@[::1],127.0.0.2/?replicaSet=replName",
      "user",
@@ -210,9 +253,18 @@ const URITestCase validCases[] = {
      "replName",
      2,
      1,
-     ""},
+     "",
+     kGlobalSSLMode},
 
-    {"mongodb://user@[::1],127.0.0.2/?replicaSet=replName", "user", "", kSet, "replName", 2, 1, ""},
+    {"mongodb://user@[::1],127.0.0.2/?replicaSet=replName",
+     "user",
+     "",
+     kSet,
+     "replName",
+     2,
+     1,
+     "",
+     kGlobalSSLMode},
 
     {"mongodb://[::1],127.0.0.2/dbName?foo=a&c=b&replicaSet=replName",
      "",
@@ -221,7 +273,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      3,
-     "dbName"},
+     "dbName",
+     kGlobalSSLMode},
 
     {"mongodb://user:pwd@[::1]:1234,127.0.0.2:1234/?replicaSet=replName",
      "user",
@@ -230,7 +283,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      1,
-     ""},
+     "",
+     kGlobalSSLMode},
 
     {"mongodb://user@[::1]:1234,127.0.0.2:1234/?replicaSet=replName",
      "user",
@@ -239,7 +293,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      1,
-     ""},
+     "",
+     kGlobalSSLMode},
 
     {"mongodb://[::1]:1234,[::1]:1234/dbName?foo=a&c=b&replicaSet=replName",
      "",
@@ -248,19 +303,20 @@ const URITestCase validCases[] = {
      "replName",
      2,
      3,
-     "dbName"},
+     "dbName",
+     kGlobalSSLMode},
 
-    {"mongodb://user:pwd@[::1]", "user", "pwd", kMaster, "", 1, 0, ""},
+    {"mongodb://user:pwd@[::1]", "user", "pwd", kMaster, "", 1, 0, "", kGlobalSSLMode},
 
-    {"mongodb://user@[::1]", "user", "", kMaster, "", 1, 0, ""},
+    {"mongodb://user@[::1]", "user", "", kMaster, "", 1, 0, "", kGlobalSSLMode},
 
-    {"mongodb://[::1]/dbName?foo=a&c=b", "", "", kMaster, "", 1, 2, "dbName"},
+    {"mongodb://[::1]/dbName?foo=a&c=b", "", "", kMaster, "", 1, 2, "dbName", kGlobalSSLMode},
 
-    {"mongodb://user:pwd@[::1]:1234", "user", "pwd", kMaster, "", 1, 0, ""},
+    {"mongodb://user:pwd@[::1]:1234", "user", "pwd", kMaster, "", 1, 0, "", kGlobalSSLMode},
 
-    {"mongodb://user@[::1]:1234", "user", "", kMaster, "", 1, 0, ""},
+    {"mongodb://user@[::1]:1234", "user", "", kMaster, "", 1, 0, "", kGlobalSSLMode},
 
-    {"mongodb://[::1]:1234/dbName?foo=a&c=b", "", "", kMaster, "", 1, 2, "dbName"},
+    {"mongodb://[::1]:1234/dbName?foo=a&c=b", "", "", kMaster, "", 1, 2, "dbName", kGlobalSSLMode},
 
     {"mongodb://user:pwd@[::1],127.0.0.2/?replicaSet=replName",
      "user",
@@ -269,9 +325,18 @@ const URITestCase validCases[] = {
      "replName",
      2,
      1,
-     ""},
+     "",
+     kGlobalSSLMode},
 
-    {"mongodb://user@[::1],127.0.0.2/?replicaSet=replName", "user", "", kSet, "replName", 2, 1, ""},
+    {"mongodb://user@[::1],127.0.0.2/?replicaSet=replName",
+     "user",
+     "",
+     kSet,
+     "replName",
+     2,
+     1,
+     "",
+     kGlobalSSLMode},
 
     {"mongodb://[::1],127.0.0.2/dbName?foo=a&c=b&replicaSet=replName",
      "",
@@ -280,7 +345,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      3,
-     "dbName"},
+     "dbName",
+     kGlobalSSLMode},
 
     {"mongodb://user:pwd@[::1]:1234,127.0.0.2:1234/?replicaSet=replName",
      "user",
@@ -289,7 +355,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      1,
-     ""},
+     "",
+     kGlobalSSLMode},
 
     {"mongodb://user@[::1]:1234,127.0.0.2:1234/?replicaSet=replName",
      "user",
@@ -298,7 +365,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      1,
-     ""},
+     "",
+     kGlobalSSLMode},
 
     {"mongodb://[::1]:1234,[::1]:1234/dbName?foo=a&c=b&replicaSet=replName",
      "",
@@ -307,7 +375,8 @@ const URITestCase validCases[] = {
      "replName",
      2,
      3,
-     "dbName"},
+     "dbName",
+     kGlobalSSLMode},
 
     {"mongodb://user:pwd@[::1]/?authMechanism=GSSAPI&authMechanismProperties=SERVICE_NAME:foobar",
      "user",
@@ -316,7 +385,8 @@ const URITestCase validCases[] = {
      "",
      1,
      2,
-     ""},
+     "",
+     kGlobalSSLMode},
 
     {"mongodb://user:pwd@[::1]/?authMechanism=GSSAPI&gssapiServiceName=foobar",
      "user",
@@ -325,9 +395,10 @@ const URITestCase validCases[] = {
      "",
      1,
      2,
-     ""},
+     "",
+     kGlobalSSLMode},
 
-    {"mongodb://%2Ftmp%2Fmongodb-27017.sock", "", "", kMaster, "", 1, 0, ""},
+    {"mongodb://%2Ftmp%2Fmongodb-27017.sock", "", "", kMaster, "", 1, 0, "", kGlobalSSLMode},
 
     {"mongodb://%2Ftmp%2Fmongodb-27017.sock,%2Ftmp%2Fmongodb-27018.sock/?replicaSet=replName",
      "",
@@ -336,7 +407,11 @@ const URITestCase validCases[] = {
      "replName",
      2,
      1,
-     ""},
+     "",
+     kGlobalSSLMode},
+
+    {"mongodb://localhost/?ssl=true", "", "", kMaster, "", 1, 1, "", kEnableSSL},
+    {"mongodb://localhost/?ssl=false", "", "", kMaster, "", 1, 1, "", kDisableSSL},
 };
 
 const InvalidURITestCase invalidCases[] = {
@@ -354,6 +429,9 @@ const InvalidURITestCase invalidCases[] = {
     // Host list must actually be comma separated.
     {"mongodb://localhost:27017localhost:27018"},
 
+    // % symbol in password must be escaped.
+    {"mongodb://localhost:pass%word@127.0.0.1:27017", ErrorCodes::duplicateCodeForTest(51040)},
+
     // Domain sockets have to end in ".sock".
     {"mongodb://%2Fnotareal%2Fdomainsock"},
 
@@ -404,6 +482,9 @@ const InvalidURITestCase invalidCases[] = {
 
     // Missing an entire key-value pair
     {"mongodb://127.0.0.1:1234/dbName?foo=a&&c=b"},
+
+    // Illegal value for ssl.
+    {"mongodb://127.0.0.1:1234/dbName?ssl=blah", ErrorCodes::duplicateCodeForTest(51041)},
 };
 
 // Helper Method to take a filename for a json file and return the array of tests inside of it
@@ -465,6 +546,9 @@ TEST(MongoURI, InvalidURIs) {
         unittest::log() << "Testing URI: " << testCase.URI << '\n';
         auto cs_status = MongoURI::parse(testCase.URI);
         ASSERT_NOT_OK(cs_status);
+        if (testCase.code) {
+            ASSERT_EQUALS(*testCase.code, cs_status.getStatus());
+        }
     }
 }
 
@@ -540,7 +624,7 @@ TEST(MongoURI, specTests) {
 
             if (!valid) {
                 // This uri string is invalid --> parse the uri and ensure it fails
-                const InvalidURITestCase testCase = {uri};
+                const InvalidURITestCase testCase = InvalidURITestCase{uri};
                 unittest::log() << "Testing URI: " << testCase.URI << '\n';
                 auto cs_status = MongoURI::parse(testCase.URI);
                 ASSERT_NOT_OK(cs_status);
