@@ -95,10 +95,13 @@ public:
           _innerInvocation{std::move(innerInvocation)} {}
 
     void run(OperationContext* opCtx, CommandReplyBuilder* result) override {
+        // Explain is never allowed in multi-document transactions.
+        const bool inMultiDocumentTransaction = false;
         uassert(50746,
                 "Explain's child command cannot run on this node. "
                 "Are you explaining a write command on a secondary?",
-                commandCanRunHere(opCtx, _dbName, _innerInvocation->definition()));
+                commandCanRunHere(
+                    opCtx, _dbName, _innerInvocation->definition(), inMultiDocumentTransaction));
         try {
             BSONObjBuilder bob = result->getBodyBuilder();
             _innerInvocation->explain(opCtx, _verbosity, &bob);
