@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2014 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -160,10 +162,8 @@ bool getS2GeoKeys(const BSONObj& document,
 /**
  * Fills 'out' with the keys that should be generated for an array value 'obj' in a 2dsphere index.
  * A key is generated for each element of the array value 'obj'.
- *
- * Returns true if 'obj' contains more than one element, and returns false otherwise.
  */
-bool getS2LiteralKeysArray(const BSONObj& obj, const CollatorInterface* collator, BSONObjSet* out) {
+void getS2LiteralKeysArray(const BSONObj& obj, const CollatorInterface* collator, BSONObjSet* out) {
     BSONObjIterator objIt(obj);
     if (!objIt.more()) {
         // Empty arrays are indexed as undefined.
@@ -172,33 +172,26 @@ bool getS2LiteralKeysArray(const BSONObj& obj, const CollatorInterface* collator
         out->insert(b.obj());
     } else {
         // Non-empty arrays are exploded.
-        size_t nArrElems = 0;
         while (objIt.more()) {
             BSONObjBuilder b;
             CollationIndexKey::collationAwareIndexKeyAppend(objIt.next(), collator, &b);
             out->insert(b.obj());
-            ++nArrElems;
-        }
-
-        if (nArrElems > 1) {
-            return true;
         }
     }
-    return false;
 }
 
 /**
  * Fills 'out' with the keys that should be generated for a value 'elt' in a 2dsphere index. If
  * 'elt' is an array value, then a key is generated for each element of the array value 'obj'.
  *
- * Returns true if 'elt' is an array value that contains more than one element, and returns false
- * otherwise.
+ * Returns true if 'elt' is an array value and returns false otherwise.
  */
 bool getS2OneLiteralKey(const BSONElement& elt,
                         const CollatorInterface* collator,
                         BSONObjSet* out) {
     if (Array == elt.type()) {
-        return getS2LiteralKeysArray(elt.Obj(), collator, out);
+        getS2LiteralKeysArray(elt.Obj(), collator, out);
+        return true;
     } else {
         // One thing, not an array, index as-is.
         BSONObjBuilder b;
@@ -213,13 +206,12 @@ bool getS2OneLiteralKey(const BSONElement& elt,
  * any element in 'elements' is an array value, then a key is generated for each element of that
  * array value.
  *
- * Returns true if any element of 'elements' is an array value that contains more than one element,
- * and returns false otherwise.
+ * Returns true if any element of 'elements' is an array value and returns false otherwise.
  */
 bool getS2LiteralKeys(const BSONElementSet& elements,
                       const CollatorInterface* collator,
                       BSONObjSet* out) {
-    bool indexedArrayValueWithMultipleElements = false;
+    bool foundIndexedArrayValue = false;
     if (0 == elements.size()) {
         // Missing fields are indexed as null.
         BSONObjBuilder b;
@@ -227,12 +219,11 @@ bool getS2LiteralKeys(const BSONElementSet& elements,
         out->insert(b.obj());
     } else {
         for (BSONElementSet::iterator i = elements.begin(); i != elements.end(); ++i) {
-            const bool thisElemIsArrayWithMultipleElements = getS2OneLiteralKey(*i, collator, out);
-            indexedArrayValueWithMultipleElements =
-                indexedArrayValueWithMultipleElements || thisElemIsArrayWithMultipleElements;
+            const bool thisElemIsArray = getS2OneLiteralKey(*i, collator, out);
+            foundIndexedArrayValue = foundIndexedArrayValue || thisElemIsArray;
         }
     }
-    return indexedArrayValueWithMultipleElements;
+    return foundIndexedArrayValue;
 }
 
 }  // namespace
@@ -246,8 +237,7 @@ using std::vector;
 // static
 void ExpressionKeysPrivate::get2DKeys(const BSONObj& obj,
                                       const TwoDIndexingParams& params,
-                                      BSONObjSet* keys,
-                                      std::vector<BSONObj>* locs) {
+                                      BSONObjSet* keys) {
     BSONElementMultiSet bSet;
 
     // Get all the nested location fields, but don't return individual elements from
@@ -301,10 +291,6 @@ void ExpressionKeysPrivate::get2DKeys(const BSONObj& obj,
 
             BSONObjBuilder b(64);
 
-            // Remember the actual location object if needed
-            if (locs)
-                locs->push_back(locObj);
-
             // Stop if we don't need to get anything but location objects
             if (!keys) {
                 if (singleElement)
@@ -359,35 +345,9 @@ void ExpressionKeysPrivate::getHashKeys(const BSONObj& obj,
                                         int hashVersion,
                                         bool isSparse,
                                         const CollatorInterface* collator,
-                                        BSONObjSet* keys,
-                                        bool ignoreArraysAlongPath) {
-    static const BSONObj nullObj = BSON("" << BSONNULL);
+                                        BSONObjSet* keys) {
     const char* cstr = hashedField.c_str();
-    BSONElement fieldVal = dps::extractElementAtPathOrArrayAlongPath(obj, cstr);
-
-    // If we hit an array while traversing the path, 'cstr' will point to the path component
-    // immediately following the array, or the null termination byte if the terminal path
-    // component was an array. In the latter case, 'remainingPath' will be empty.
-    auto remainingPath = StringData(cstr);
-
-    // If 'ignoreArraysAlongPath' is set, we want to use the behaviour prior to SERVER-44050,
-    // which is to allow arrays along the field path (except the terminal path). This is done so
-    // that the document keys inserted prior to SERVER-44050 can be deleted or updated after the
-    // upgrade, allowing users to recover from the possible index corruption. The old behaviour
-    // before SERVER-44050 was to store 'null' index key if we encountered an array along the
-    // index field path. We will use the same logic in the context of removing index keys.
-    if (ignoreArraysAlongPath && fieldVal.type() == BSONType::Array && !remainingPath.empty()) {
-        fieldVal = nullObj.firstElement();
-    }
-
-    // Otherwise, throw if an array was encountered at any point along the path.
-    uassert(16766,
-            str::stream() << "Error: hashed indexes do not currently support array values. "
-                             "Found array at path: "
-                          << hashedField.substr(0,
-                                                hashedField.size() - remainingPath.size() -
-                                                    !remainingPath.empty()),
-            fieldVal.type() != BSONType::Array);
+    BSONElement fieldVal = dps::extractElementAtPath(obj, cstr);
 
     // Convert strings to comparison keys.
     BSONObj fieldValObj;
@@ -396,9 +356,17 @@ void ExpressionKeysPrivate::getHashKeys(const BSONObj& obj,
         CollationIndexKey::collationAwareIndexKeyAppend(fieldVal, collator, &bob);
         fieldValObj = bob.obj();
         fieldVal = fieldValObj.firstElement();
+    }
+
+    uassert(16766,
+            "Error: hashed indexes do not currently support array values",
+            fieldVal.type() != Array);
+
+    if (!fieldVal.eoo()) {
         BSONObj key = BSON("" << makeSingleHashKey(fieldVal, seed, hashVersion));
         keys->insert(key);
     } else if (!isSparse) {
+        BSONObj nullObj = BSON("" << BSONNULL);
         keys->insert(BSON("" << makeSingleHashKey(nullObj.firstElement(), seed, hashVersion)));
     }
 }
@@ -505,8 +473,8 @@ void ExpressionKeysPrivate::getS2Keys(const BSONObj& obj,
         // Trailing array values aren't being expanded, so we still need to determine whether the
         // last component of the indexed path 'keyElem.fieldName()' causes the index to be multikey.
         // We say that it does if
-        //   (a) the last component of the indexed path ever refers to an array value containing
-        //       multiple elements, or if
+        //   (a) the last component of the indexed path ever refers to an array value (regardless of
+        //       the number of array elements)
         //   (b) the last component of the indexed path ever refers to GeoJSON data that requires
         //       multiple cells for its covering.
         bool lastPathComponentCausesIndexToBeMultikey;
