@@ -146,7 +146,23 @@ struct QuerySolutionNode {
         }
     }
 
+    /**
+     * Adds a vector of query solution nodes to the list of children of this node.
+     *
+     * TODO SERVER-35512: Once 'children' are held by unique_ptr, this method should no longer be
+     * necessary.
+     */
+    void addChildren(std::vector<std::unique_ptr<QuerySolutionNode>> newChildren) {
+        children.reserve(children.size() + newChildren.size());
+        std::transform(newChildren.begin(),
+                       newChildren.end(),
+                       std::back_inserter(children),
+                       [](auto& child) { return child.release(); });
+    }
+
     // These are owned here.
+    //
+    // TODO SERVER-35512: Make this a vector of unique_ptr.
     std::vector<QuerySolutionNode*> children;
 
     // If a stage has a non-NULL filter all values outputted from that stage must pass that
