@@ -182,6 +182,11 @@ public:
      */
     virtual void setGlobalTimestamp(ServiceContext* service, const Timestamp& newTime) = 0;
 
+    /**
+     * Gets the global opTime timestamp, i.e. the latest cluster time.
+     */
+    virtual Timestamp getGlobalTimestamp(ServiceContext* service) = 0;
+
     /**
      * Checks if the oplog exists.
      */
