@@ -707,9 +707,12 @@ Status storeServerOptions(const moe::Environment& params) {
     }
 #endif
 
-    ret = storeMessageCompressionOptions(params);
-    if (!ret.isOK()) {
-        return ret;
+    if (params.count("net.compression.compressors")) {
+        const auto ret =
+            storeMessageCompressionOptions(params["net.compression.compressors"].as<string>());
+        if (!ret.isOK()) {
+            return ret;
+        }
     }
 
     return Status::OK();
