@@ -50,6 +50,7 @@
 #include "mongo/db/query/get_executor.h"
 #include "mongo/db/repl/read_concern_args.h"
 #include "mongo/db/repl/repl_client_info.h"
+#include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/retryable_writes_stats.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/server_parameters.h"
@@ -388,9 +389,9 @@ void Session::beginOrContinueTxnOnMigration(OperationContext* opCtx, TxnNumber t
     _beginOrContinueTxnOnMigration(lg, txnNumber);
 }
 
-void Session::setSpeculativeTransactionOpTime(OperationContext* opCtx,
-                                              SpeculativeTransactionOpTime opTimeChoice) {
-    stdx::lock_guard<stdx::mutex> lg(_mutex);
+void Session::_setSpeculativeTransactionOpTime(WithLock,
+                                               OperationContext* opCtx,
+                                               SpeculativeTransactionOpTime opTimeChoice) {
     repl::ReplicationCoordinator* replCoord =
         repl::ReplicationCoordinator::get(opCtx->getClient()->getServiceContext());
     opCtx->recoveryUnit()->setTimestampReadSource(
@@ -398,13 +399,12 @@ void Session::setSpeculativeTransactionOpTime(OperationContext* opCtx,
             ? RecoveryUnit::ReadSource::kAllCommittedSnapshot
             : RecoveryUnit::ReadSource::kLastAppliedSnapshot);
     opCtx->recoveryUnit()->preallocateSnapshot();
-    auto readTimestamp = opCtx->recoveryUnit()->getPointInTimeReadTimestamp();
-    invariant(readTimestamp);
+    auto readTimestamp = repl::StorageInterface::get(opCtx)->getPointInTimeReadTimestamp(opCtx);
     // Transactions do not survive term changes, so combining "getTerm" here with the
     // recovery unit timestamp does not cause races.
-    _speculativeTransactionReadOpTime = {*readTimestamp, replCoord->getTerm()};
+    _speculativeTransactionReadOpTime = {readTimestamp, replCoord->getTerm()};
     stdx::lock_guard<stdx::mutex> ls(_statsMutex);
-    _singleTransactionStats.setReadTimestamp(*readTimestamp);
+    _singleTransactionStats.setReadTimestamp(readTimestamp);
 }
 
 void Session::onWriteOpCompletedOnPrimary(OperationContext* opCtx,
@@ -445,18 +445,17 @@ bool Session::onMigrateBeginOnPrimary(OperationContext* opCtx, TxnNumber txnNumb
             return false;
         }
     } catch (const DBException& ex) {
-        // If the transaction chain was truncated on the recipient shard, then we are most likely
-        // copying from a session that hasn't been touched on the recipient shard for a very long
-        // time but could be recent on the donor.
-        //
-        // We continue copying regardless to get the entire transaction from the donor.
-        if (ex.code() != ErrorCodes::IncompleteTransactionHistory) {
-            throw;
+        // If the transaction chain is incomplete because oplog was truncated, just ignore the
+        // incoming oplog and don't attempt to 'patch up' the missing pieces.
+        if (ex.code() == ErrorCodes::IncompleteTransactionHistory) {
+            return false;
         }
 
         if (stmtId == kIncompleteHistoryStmtId) {
             return false;
         }
+
+        throw;
     }
 
     return true;
@@ -582,10 +581,18 @@ void Session::_beginOrContinueTxn(WithLock wl,
                 // This indicates that the first command in the transaction failed but did not
                 // implicitly abort the transaction. It is not safe to continue the transaction, in
                 // particular because we have not saved the readConcern from the first statement of
-                // the transaction.
+                // the transaction. Mark the transaction as active here, since
+                // _abortTransaction() will assume we are aborting an active transaction since there
+                // are no stashed resources.
+                _singleTransactionStats.setActive(curTimeMicros64());
+                ServerTransactionsMetrics::get(getGlobalServiceContext())->incrementCurrentActive();
+                ServerTransactionsMetrics::get(getGlobalServiceContext())
+                    ->decrementCurrentInactive();
                 _abortTransaction(wl);
                 uasserted(ErrorCodes::NoSuchTransaction,
-                          str::stream() << "Transaction " << txnNumber << " has been aborted.");
+                          str::stream() << "Transaction " << txnNumber
+                                        << " has been aborted because an earlier command in this "
+                                           "transaction failed.");
             }
         }
         return;
@@ -623,7 +630,7 @@ void Session::_beginOrContinueTxn(WithLock wl,
 
         const auto now = curTimeMicros64();
         _transactionExpireDate = Date_t::fromMillisSinceEpoch(now / 1000) +
-            stdx::chrono::seconds{transactionLifetimeLimitSeconds.load()};
+            Seconds{transactionLifetimeLimitSeconds.load()};
         // Tracks various transactions metrics.
         {
             stdx::lock_guard<stdx::mutex> ls(_statsMutex);
@@ -814,14 +821,28 @@ void Session::unstashTransactionResources(OperationContext* opCtx, const std::st
             return;
         }
 
-        // Stashed transaction resources do not exist for this transaction.  If this is a
-        // multi-document transaction, set up the transaction resources on the opCtx.
+        // Stashed transaction resources do not exist for this transaction.
         if (_txnState != MultiDocumentTransactionState::kInProgress) {
             return;
         }
+
+        // Set speculative execution.
+        const auto& readConcernArgs = repl::ReadConcernArgs::get(opCtx);
+        const bool speculative =
+            readConcernArgs.getLevel() == repl::ReadConcernLevel::kSnapshotReadConcern &&
+            !readConcernArgs.getArgsAtClusterTime();
+        // Only set speculative on primary.
+        if (opCtx->writesAreReplicated() && speculative) {
+            _setSpeculativeTransactionOpTime(lg,
+                                             opCtx,
+                                             readConcernArgs.getOriginalLevel() ==
+                                                     repl::ReadConcernLevel::kSnapshotReadConcern
+                                                 ? SpeculativeTransactionOpTime::kAllCommitted
+                                                 : SpeculativeTransactionOpTime::kLastApplied);
+        }
+
+        // If this is a multi-document transaction, set up the transaction resources on the opCtx.
         opCtx->setWriteUnitOfWork(std::make_unique<WriteUnitOfWork>(opCtx));
-        ServerTransactionsMetrics::get(getGlobalServiceContext())->incrementCurrentActive();
-        ServerTransactionsMetrics::get(getGlobalServiceContext())->decrementCurrentInactive();
 
         // If maxTransactionLockRequestTimeoutMillis is set, then we will ensure no
         // future lock request waits longer than maxTransactionLockRequestTimeoutMillis
@@ -831,10 +852,6 @@ void Session::unstashTransactionResources(OperationContext* opCtx, const std::st
         if (maxTransactionLockMillis >= 0) {
             opCtx->lockState()->setMaxLockTimeout(Milliseconds(maxTransactionLockMillis));
         }
-
-        // Set the starting active time for this transaction.
-        stdx::lock_guard<stdx::mutex> ls(_statsMutex);
-        _singleTransactionStats.setActive(curTimeMicros64());
     }
 
     // Storage engine transactions may be started in a lazy manner. By explicitly
@@ -854,6 +871,14 @@ void Session::unstashTransactionResources(OperationContext* opCtx, const std::st
         CurOpFailpointHelpers::waitWhileFailPointEnabled(
             &hangAfterPreallocateSnapshot, opCtx, "hangAfterPreallocateSnapshot");
     }
+
+    {
+        stdx::lock_guard<stdx::mutex> lg(_mutex);
+        stdx::lock_guard<stdx::mutex> ls(_statsMutex);
+        _singleTransactionStats.setActive(curTimeMicros64());
+        ServerTransactionsMetrics::get(getGlobalServiceContext())->incrementCurrentActive();
+        ServerTransactionsMetrics::get(getGlobalServiceContext())->decrementCurrentInactive();
+    }
 }
 
 void Session::abortArbitraryTransaction() {
