@@ -454,6 +454,10 @@ KVStorageEngine::reconcileCatalogAndIdents(OperationContext* opCtx) {
     return ret;
 }
 
+std::string KVStorageEngine::getFilesystemPathForDb(const std::string& dbName) const {
+    return _catalog->getFilesystemPathForDb(dbName);
+}
+
 void KVStorageEngine::cleanShutdown() {
     for (DBMap::const_iterator it = _dbs.begin(); it != _dbs.end(); ++it) {
         delete it->second;
@@ -694,6 +698,10 @@ Timestamp KVStorageEngine::getAllCommittedTimestamp() const {
     return _engine->getAllCommittedTimestamp();
 }
 
+Timestamp KVStorageEngine::getOldestOpenReadTimestamp() const {
+    return _engine->getOldestOpenReadTimestamp();
+}
+
 void KVStorageEngine::_dumpCatalog(OperationContext* opCtx) {
     auto catalogRs = _catalogRecordStore.get();
     auto cursor = catalogRs->getCursor(opCtx);
