@@ -214,6 +214,8 @@ public:
 
     bool supportsReadConcernSnapshot() const final;
 
+    virtual Timestamp getOldestOpenReadTimestamp() const override;
+
     bool supportsReadConcernMajority() const final;
 
     // wiredtiger specific
@@ -282,6 +284,8 @@ public:
 
     static void appendGlobalStats(BSONObjBuilder& b);
 
+    Timestamp getOldestTimestamp() const;
+
     /**
      * Returns the data file path associated with an ident on disk. Returns boost::none if the data
      * file can not be found. This will attempt to locate a file even if the storage engine's own
