@@ -85,9 +85,8 @@ RoutingTableHistory::RoutingTableHistory(NamespaceString nss,
       _defaultCollator(std::move(defaultCollator)),
       _unique(unique),
       _chunkMap(std::move(chunkMap)),
-      _shardVersions(
-          _constructShardVersionMap(collectionVersion.epoch(), _chunkMap, _shardKeyOrdering)),
-      _collectionVersion(collectionVersion) {}
+      _collectionVersion(collectionVersion),
+      _shardVersions(_constructShardVersionMap()) {}
 
 Chunk ChunkManager::findIntersectingChunk(const BSONObj& shardKey, const BSONObj& collation) const {
     const bool hasSimpleCollation = (collation.isEmpty() && !_rt->getDefaultCollator()) ||
@@ -399,66 +398,65 @@ std::string RoutingTableHistory::toString() const {
     return sb.str();
 }
 
-ShardVersionMap RoutingTableHistory::_constructShardVersionMap(const OID& epoch,
-                                                               const ChunkInfoMap& chunkMap,
-                                                               Ordering shardKeyOrdering) {
+ShardVersionMap RoutingTableHistory::_constructShardVersionMap() const {
+    const OID& epoch = _collectionVersion.epoch();
+
     ShardVersionMap shardVersions;
-    ChunkInfoMap::const_iterator current = chunkMap.cbegin();
+    ChunkInfoMap::const_iterator current = _chunkMap.cbegin();
 
     boost::optional<BSONObj> firstMin = boost::none;
     boost::optional<BSONObj> lastMax = boost::none;
 
-    while (current != chunkMap.cend()) {
+    while (current != _chunkMap.cend()) {
         const auto& firstChunkInRange = current->second;
+        const auto& currentRangeShardId = firstChunkInRange->getShardIdAt(boost::none);
 
         // Tracks the max shard version for the shard on which the current range will reside
-        auto shardVersionIt = shardVersions.find(firstChunkInRange->getShardIdAt(boost::none));
+        auto shardVersionIt = shardVersions.find(currentRangeShardId);
         if (shardVersionIt == shardVersions.end()) {
-            shardVersionIt = shardVersions
-                                 .emplace(firstChunkInRange->getShardIdAt(boost::none),
-                                          ChunkVersion(0, 0, epoch))
-                                 .first;
+            shardVersionIt =
+                shardVersions.emplace(currentRangeShardId, ChunkVersion(0, 0, epoch)).first;
         }
 
         auto& maxShardVersion = shardVersionIt->second;
 
-        current = std::find_if(
-            current,
-            chunkMap.cend(),
-            [&firstChunkInRange, &maxShardVersion](const ChunkInfoMap::value_type& chunkMapEntry) {
-                const auto& currentChunk = chunkMapEntry.second;
+        current =
+            std::find_if(current,
+                         _chunkMap.cend(),
+                         [&currentRangeShardId,
+                          &maxShardVersion](const ChunkInfoMap::value_type& chunkMapEntry) {
+                             const auto& currentChunk = chunkMapEntry.second;
 
-                if (currentChunk->getShardIdAt(boost::none) !=
-                    firstChunkInRange->getShardIdAt(boost::none))
-                    return true;
+                             if (currentChunk->getShardIdAt(boost::none) != currentRangeShardId)
+                                 return true;
 
-                if (currentChunk->getLastmod() > maxShardVersion)
-                    maxShardVersion = currentChunk->getLastmod();
+                             if (currentChunk->getLastmod() > maxShardVersion)
+                                 maxShardVersion = currentChunk->getLastmod();
 
-                return false;
-            });
+                             return false;
+                         });
 
         const auto rangeLast = std::prev(current);
 
-        const BSONObj rangeMin = firstChunkInRange->getMin();
-        const BSONObj rangeMax = rangeLast->second->getMax();
+        const auto& rangeMin = firstChunkInRange->getMin();
+        const auto& rangeMax = rangeLast->second->getMax();
 
-        if (lastMax) {
-            uassert(ErrorCodes::ConflictingOperationInProgress,
-                    str::stream()
-                        << "Metadata contains chunks with the same or out-of-order max value; "
-                           "expected "
-                        << lastMax.get()
-                        << " < "
-                        << rangeMax,
-                    SimpleBSONObjComparator::kInstance.evaluate(lastMax.get() < rangeMax));
-            // Make sure there are no gaps in the ranges
-            uassert(ErrorCodes::ConflictingOperationInProgress,
-                    str::stream() << "Gap or an overlap between ranges "
-                                  << ChunkRange(rangeMin, rangeMax).toString()
-                                  << " and "
-                                  << lastMax.get(),
-                    SimpleBSONObjComparator::kInstance.evaluate(lastMax.get() == rangeMin));
+        // Check the continuity of the chunks map
+        if (lastMax && !SimpleBSONObjComparator::kInstance.evaluate(*lastMax == rangeMin)) {
+            if (SimpleBSONObjComparator::kInstance.evaluate(*lastMax < rangeMin))
+                uasserted(ErrorCodes::ConflictingOperationInProgress,
+                          str::stream()
+                              << "Gap exists in the routing table between chunks "
+                              << _chunkMap.at(_extractKeyString(*lastMax))->getRange().toString()
+                              << " and "
+                              << rangeLast->second->getRange().toString());
+            else
+                uasserted(ErrorCodes::ConflictingOperationInProgress,
+                          str::stream()
+                              << "Overlap exists in the routing table between chunks "
+                              << _chunkMap.at(_extractKeyString(*lastMax))->getRange().toString()
+                              << " and "
+                              << rangeLast->second->getRange().toString());
         }
 
         if (!firstMin)
@@ -471,7 +469,7 @@ ShardVersionMap RoutingTableHistory::_constructShardVersionMap(const OID& epoch,
         invariant(maxShardVersion.isSet());
     }
 
-    if (!chunkMap.empty()) {
+    if (!_chunkMap.empty()) {
         invariant(!shardVersions.empty());
         invariant(firstMin.is_initialized());
         invariant(lastMax.is_initialized());
