@@ -1,5 +1,8 @@
 /**
  * Implements the sessions api for the shell.
+ *
+ * Roughly follows the driver sessions spec:
+ * https://github.com/mongodb/specifications/blob/master/source/sessions/driver-sessions.rst#abstract
  */
 var {
     DriverSession, SessionOptions, _DummyDriverSession, _DelegatingDriverSession,
@@ -80,11 +83,12 @@ var {
         };
     }
 
-    function SessionAwareClient(client) {
-        const kWireVersionSupportingCausalConsistency = 6;
-        const kWireVersionSupportingLogicalSession = 6;
-        const kWireVersionSupportingRetryableWrites = 6;
+    const kWireVersionSupportingCausalConsistency = 6;
+    const kWireVersionSupportingLogicalSession = 6;
+    const kWireVersionSupportingRetryableWrites = 6;
+    const kWireVersionSupportingMultiDocumentTransactions = 7;
 
+    function SessionAwareClient(client) {
         this.getReadPreference = function getReadPreference(driverSession) {
             const sessionOptions = driverSession.getOptions();
             if (sessionOptions.getReadPreference() !== undefined) {
@@ -133,7 +137,13 @@ var {
             "parallelCollectionScan",
         ]);
 
-        function canUseReadConcern(cmdObj) {
+        function canUseReadConcern(driverSession, cmdObj) {
+            // Always attach the readConcern to the first statement of the transaction, whether it
+            // is a read or a write.
+            if (driverSession._serverSession.isTxnActive()) {
+                return driverSession._serverSession.isFirstStatement();
+            }
+
             let cmdName = Object.keys(cmdObj)[0];
 
             // If the command is in a wrapped form, then we look for the actual command name inside
@@ -199,7 +209,14 @@ var {
         }
 
         function prepareCommandRequest(driverSession, cmdObj) {
-            if (serverSupports(kWireVersionSupportingLogicalSession)) {
+            if (serverSupports(kWireVersionSupportingLogicalSession) &&
+                // Always attach sessionId from explicit sessions.
+                (driverSession._isExplicit ||
+                 // Check that implicit sessions are not disabled. The client must be using read
+                 // commands because aggregations always use runCommand() to establish cursors but
+                 // may use OP_GET_MORE (and therefore not have a session id attached) to retrieve
+                 // subsequent batches.
+                 (!jsTest.options().disableImplicitSessions && client.useReadCommands()))) {
                 cmdObj = driverSession._serverSession.injectSessionId(cmdObj);
             }
 
@@ -248,7 +265,7 @@ var {
                 (client.isReplicaSetMember() || client.isMongos()) &&
                 (driverSession.getOptions().isCausalConsistency() ||
                  client.isCausalConsistency()) &&
-                canUseReadConcern(cmdObj)) {
+                canUseReadConcern(driverSession, cmdObj)) {
                 // `driverSession.getOperationTime()` is the smallest time needed for performing a
                 // causally consistent read using the current session. Note that
                 // `client.getClusterTime()` is no smaller than the operation time and would
@@ -259,6 +276,11 @@ var {
                 }
             }
 
+            // All commands go through transaction code, which will determine if the command is a
+            // part of the current transaction and will assign transaction info accordingly.
+            cmdObj = driverSession._serverSession.assignTxnInfo(cmdObj);
+
+            // Retryable writes code should execute only we are not in an active transaction.
             if (jsTest.options().alwaysInjectTransactionNumber &&
                 serverSupports(kWireVersionSupportingRetryableWrites) &&
                 driverSession.getOptions().shouldRetryWrites() &&
@@ -306,8 +328,13 @@ var {
                 cmdName = Object.keys(cmdObj)[0];
             }
 
+            // TODO SERVER-33921: Revisit how the mongo shell decides whether it should retry a
+            // command or not.
+            const sessionOptions = driverSession.getOptions();
             let numRetries =
-                (cmdObj.hasOwnProperty("txnNumber") && !jsTest.options().skipRetryOnNetworkError)
+                (sessionOptions.shouldRetryWrites() && cmdObj.hasOwnProperty("txnNumber") &&
+                 !jsTest.options().skipRetryOnNetworkError &&
+                 !driverSession._serverSession.isTxnActive())
                 ? 1
                 : 0;
 
@@ -432,17 +459,75 @@ var {
         };
     }
 
+    function TransactionOptions(rawOptions = {}) {
+        if (!(this instanceof TransactionOptions)) {
+            return new TransactionOptions(rawOptions);
+        }
+
+        let _readConcern = rawOptions.readConcern;
+        let _writeConcern = rawOptions.writeConcern;
+
+        this.setTxnReadConcern = function setTxnReadConcern(value) {
+            _readConcern = value;
+        };
+
+        this.getTxnReadConcern = function getTxnReadConcern() {
+            return _readConcern;
+        };
+
+        this.setTxnWriteConcern = function setTxnWriteConcern(value) {
+            _writeConcern = value;
+        };
+
+        this.getTxnWriteConcern = function getTxnWriteConcern() {
+            return _writeConcern;
+        };
+    }
+
+    // The server session maintains the state of a transaction, a monotonically increasing txn
+    // number, and a transaction's read/write concerns.
     function ServerSession(client) {
+        // The default txnState is `inactive` until we call startTransaction.
+        let _txnState = ServerSession.TransactionStates.kInactive;
+
+        let _txnOptions;
+
+        // Keep track of the next available statement id of a transaction.
+        let _nextStatementId = 0;
+
+        // _txnNumber starts at -1 because when we increment it, the first transaction
+        // and retryable write will both have a txnNumber of 0.
+        let _txnNumber = -1;
         let _lastUsed = new Date();
-        let _nextTxnNum = 0;
 
         this.client = new SessionAwareClient(client);
+        if (!serverSupports(kWireVersionSupportingLogicalSession)) {
+            throw new DriverSession.UnsupportedError(
+                "Logical Sessions are only supported on server versions 3.6 and greater.");
+        }
         this.handle = client._startSession();
 
+        function serverSupports(wireVersion) {
+            return client.getMinWireVersion() <= wireVersion &&
+                wireVersion <= client.getMaxWireVersion();
+        }
+
+        this.isTxnActive = function isTxnActive() {
+            return _txnState === ServerSession.TransactionStates.kActive;
+        };
+
+        this.isFirstStatement = function isFirstStatement() {
+            return _nextStatementId === 0;
+        };
+
         this.getLastUsed = function getLastUsed() {
             return _lastUsed;
         };
 
+        this.getTxnOptions = function getTxnOptions() {
+            return _txnOptions;
+        };
+
         function updateLastUsed() {
             _lastUsed = new Date();
         }
@@ -488,8 +573,8 @@ var {
                 // Since there's no native support for adding NumberLong instances and getting back
                 // another NumberLong instance, converting from a 64-bit floating-point value to a
                 // 64-bit integer value will overflow at 2**53.
-                cmdObjUnwrapped.txnNumber = new NumberLong(_nextTxnNum);
-                ++_nextTxnNum;
+                _txnNumber++;
+                cmdObjUnwrapped.txnNumber = new NumberLong(_txnNumber);
             }
 
             return cmdObj;
@@ -575,10 +660,181 @@ var {
 
             return false;
         };
+
+        this.assignTxnInfo = function assignTxnInfo(cmdObj) {
+            // We will want to reset the transaction state to 'inactive' if a normal operation
+            // follows a committed or aborted transaction.
+            if ((_txnState === ServerSession.TransactionStates.kAborted) ||
+                (_txnState === ServerSession.TransactionStates.kCommitted &&
+                 Object.keys(cmdObj)[0] !== "commitTransaction")) {
+                _txnState = ServerSession.TransactionStates.kInactive;
+            }
+
+            // If we're not in an active transaction or performing a retry on commitTransaction,
+            // return early.
+            if (_txnState === ServerSession.TransactionStates.kInactive) {
+                return cmdObj;
+            }
+
+            // If we reconnect to a 3.6 server in the middle of a transaction, we
+            // catch it here.
+            if (!serverSupports(kWireVersionSupportingMultiDocumentTransactions)) {
+                _txnState = ServerSession.TransactionStates.kInactive;
+                throw new Error(
+                    "Transactions are only supported on server versions 4.0 and greater.");
+            }
+            cmdObj = Object.assign({}, cmdObj);
+
+            const cmdName = Object.keys(cmdObj)[0];
+
+            // If the command is in a wrapped form, then we look for the actual command object
+            // inside the query/$query object.
+            let cmdObjUnwrapped = cmdObj;
+            if (cmdName === "query" || cmdName === "$query") {
+                cmdObj[cmdName] = Object.assign({}, cmdObj[cmdName]);
+                cmdObjUnwrapped = cmdObj[cmdName];
+            }
+
+            if (!cmdObjUnwrapped.hasOwnProperty("txnNumber")) {
+                // Since there's no native support for adding NumberLong instances and getting back
+                // another NumberLong instance, converting from a 64-bit floating-point value to a
+                // 64-bit integer value will overflow at 2**53.
+                cmdObjUnwrapped.txnNumber = new NumberLong(_txnNumber);
+            }
+
+            // All operations of a multi-statement transaction must specify autocommit=false.
+            cmdObjUnwrapped.autocommit = false;
+
+            // Statement Id is required on all transaction operations.
+            cmdObjUnwrapped.stmtId = new NumberInt(_nextStatementId);
+
+            // 'readConcern' and 'startTransaction' can only be specified on the first statement
+            // in a transaction.
+            if (_nextStatementId == 0) {
+                cmdObjUnwrapped.startTransaction = true;
+                if (_txnOptions.getTxnReadConcern() !== undefined) {
+                    // Override the readConcern with the one specified during startTransaction.
+                    cmdObjUnwrapped.readConcern = Object.assign(
+                        {}, cmdObjUnwrapped.readConcern, _txnOptions.getTxnReadConcern());
+                }
+            }
+
+            // Reserve the statement ids for batch writes.
+            switch (cmdName) {
+                case "insert":
+                    _nextStatementId += cmdObjUnwrapped.documents.length;
+                    break;
+                case "update":
+                    _nextStatementId += cmdObjUnwrapped.updates.length;
+                    break;
+                case "delete":
+                    _nextStatementId += cmdObjUnwrapped.deletes.length;
+                    break;
+                default:
+                    _nextStatementId += 1;
+            }
+
+            return cmdObj;
+        };
+
+        this.startTransaction = function startTransaction(txnOptsObj) {
+            // If the session is already in a transaction, raise an error.
+            if (this.isTxnActive()) {
+                throw new Error("Transaction already in progress on this session.");
+            }
+            if (!serverSupports(kWireVersionSupportingMultiDocumentTransactions)) {
+                throw new Error(
+                    "Transactions are only supported on server versions 4.0 and greater.");
+            }
+            _txnOptions = new TransactionOptions(txnOptsObj);
+            _txnState = ServerSession.TransactionStates.kActive;
+            _nextStatementId = 0;
+            _txnNumber++;
+        };
+
+        this.commitTransaction = function commitTransaction(driverSession) {
+            // If the transaction state is already 'aborted' we cannot try to commit it.
+            if (_txnState === ServerSession.TransactionStates.kAborted) {
+                throw new Error("Cannot call commitTransaction after calling abortTransaction.");
+            }
+            // If the session has no active transaction, raise an error.
+            if (_txnState === ServerSession.TransactionStates.kInactive) {
+                throw new Error("There is no active transaction to commit on this session.");
+            }
+            // run commitTxn command
+            return endTransaction("commitTransaction", driverSession);
+        };
+
+        this.abortTransaction = function abortTransaction(driverSession) {
+            // If the transaction state is already 'aborted' we cannot try to abort it again.
+            if (_txnState === ServerSession.TransactionStates.kAborted) {
+                throw new Error("Cannot call abortTransaction twice.");
+            }
+            // We cannot attempt to abort a transaction that has already been committed.
+            if (_txnState === ServerSession.TransactionStates.kCommitted) {
+                throw new Error("Cannot call abortTransaction after calling commitTransaction.");
+            }
+            // If the session has no active transaction, raise an error.
+            if (_txnState === ServerSession.TransactionStates.kInactive) {
+                throw new Error("There is no active transaction to abort on this session.");
+            }
+            // run abortTxn command
+            return endTransaction("abortTransaction", driverSession);
+        };
+
+        const endTransaction = (commandName, driverSession) => {
+            // If commitTransaction or abortTransaction is the first statement in a
+            // transaction, it should not send a command to the server and should mark the
+            // transaction as 'committed' or 'aborted' accordingly.
+            if (this.isFirstStatement()) {
+                if (commandName === "commitTransaction") {
+                    _txnState = ServerSession.TransactionStates.kCommitted;
+                } else {
+                    _txnState = ServerSession.TransactionStates.kAborted;
+                }
+                return {"ok": 1};
+            }
+
+            let cmd = {[commandName]: 1, txnNumber: NumberLong(_txnNumber)};
+            // writeConcern should only be specified on commit or abort. If a writeConcern is
+            // not specified from the default transaction options, it will be inherited from
+            // the session.
+            if (this.client.getWriteConcern(driverSession) !== undefined) {
+                cmd.writeConcern = this.client.getWriteConcern(driverSession);
+            }
+            if (_txnOptions.getTxnWriteConcern() !== undefined) {
+                cmd.writeConcern = _txnOptions.getTxnWriteConcern();
+            }
+
+            // If commit or abort raises an error, the transaction's state should still change.
+            let res;
+            try {
+                // run command against the admin database.
+                res = this.client.runCommand(driverSession, "admin", cmd, 0);
+            } finally {
+                if (commandName === "commitTransaction") {
+                    _txnState = ServerSession.TransactionStates.kCommitted;
+                } else {
+                    _txnState = ServerSession.TransactionStates.kAborted;
+                }
+            }
+            return res;
+        };
     }
 
+    // TransactionStates represents the state of the current transaction. The default state
+    // is 'inactive' until startTransaction is called and changes the state to 'active'.
+    // Calling abortTransaction or commitTransaction will change the state to 'aborted' or
+    // 'committed' respectively, even on error.
+    ServerSession.TransactionStates = {
+        kActive: 'active',
+        kInactive: 'inactive',
+        kCommitted: 'committed',
+        kAborted: 'aborted',
+    };
+
     function makeDriverSessionConstructor(implMethods, defaultOptions = {}) {
-        return function(client, options = defaultOptions) {
+        var driverSessionConstructor = function(client, options = defaultOptions) {
             let _options = options;
             let _hasEnded = false;
 
@@ -591,6 +847,8 @@ var {
 
             this._serverSession = implMethods.createServerSession(client);
 
+            this._isExplicit = true;
+
             this.getClient = function getClient() {
                 return client;
             };
@@ -675,7 +933,40 @@ var {
                 }
                 return "session " + tojson(sessionId);
             };
+
+            this.startTransaction = function startTransaction(txnOptsObj = {}) {
+                this._serverSession.startTransaction(txnOptsObj);
+            };
+
+            this.commitTransaction = function commitTransaction() {
+                assert.commandWorked(this._serverSession.commitTransaction(this));
+            };
+
+            this.abortTransaction = function abortTransaction() {
+                // Intentionally ignore command result.
+                this._serverSession.abortTransaction(this);
+            };
+            this.commitTransaction_forTesting = function commitTransaction_forTesting() {
+                return this._serverSession.commitTransaction(this);
+            };
+
+            this.abortTransaction_forTesting = function abortTransaction_forTesting() {
+                return this._serverSession.abortTransaction(this);
+            };
         };
+
+        // Having a specific Error for when logical sessions aren't supported by the server, allows
+        // the correct fallback behavior in this case (while propagating other errors).
+        driverSessionConstructor.UnsupportedError = function(message) {
+            this.name = "DriverSession.UnsupportedError";
+            this.message = message;
+            this.stack = this.toString() + "\n" + (new Error()).stack;
+        };
+        driverSessionConstructor.UnsupportedError.prototype = Object.create(Error.prototype);
+        driverSessionConstructor.UnsupportedError.prototype.constructor =
+            driverSessionConstructor.UnsupportedError;
+
+        return driverSessionConstructor;
     }
 
     const DriverSession = makeDriverSessionConstructor({
@@ -743,6 +1034,37 @@ var {
                       canRetryWrites: function canRetryWrites(cmdObj) {
                           return false;
                       },
+
+                      assignTxnInfo: function assignTxnInfo(cmdObj) {
+                          return cmdObj;
+                      },
+
+                      isTxnActive: function isTxnActive() {
+                          return false;
+                      },
+
+                      isFirstStatement: function isFirstStatement() {
+                          return false;
+                      },
+
+                      getTxnOptions: function getTxnOptions() {
+                          return {};
+                      },
+
+                      startTransaction: function startTransaction() {
+                          throw new Error("Must call startSession() on the Mongo connection " +
+                                          "object before starting a transaction.");
+                      },
+
+                      commitTransaction: function commitTransaction() {
+                          throw new Error("Must call startSession() on the Mongo connection " +
+                                          "object before committing a transaction.");
+                      },
+
+                      abortTransaction: function abortTransaction() {
+                          throw new Error("Must call startSession() on the Mongo connection " +
+                                          "object before aborting a transaction.");
+                      },
                   };
               },
 
