@@ -1,23 +1,25 @@
+
 /**
- *    Copyright (C) 2017 MongoDB Inc.
+ *    Copyright (C) 2018-present MongoDB, Inc.
  *
- *    This program is free software: you can redistribute it and/or  modify
- *    it under the terms of the GNU Affero General Public License, version 3,
- *    as published by the Free Software Foundation.
+ *    This program is free software: you can redistribute it and/or modify
+ *    it under the terms of the Server Side Public License, version 1,
+ *    as published by MongoDB, Inc.
  *
  *    This program is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *    GNU Affero General Public License for more details.
+ *    Server Side Public License for more details.
  *
- *    You should have received a copy of the GNU Affero General Public License
- *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *    You should have received a copy of the Server Side Public License
+ *    along with this program. If not, see
+ *    <http://www.mongodb.com/licensing/server-side-public-license>.
  *
  *    As a special exception, the copyright holders give permission to link the
  *    code of portions of this program with the OpenSSL library under certain
  *    conditions as described in each individual source file and distribute
  *    linked combinations including the program with the OpenSSL library. You
- *    must comply with the GNU Affero General Public License in all respects for
+ *    must comply with the Server Side Public License in all respects for
  *    all of the code used other than as permitted herein. If you modify file(s)
  *    with this exception, you may extend this exception to your version of the
  *    file(s), but you are not obligated to do so. If you do not wish to do so,
@@ -63,13 +65,15 @@ public:
         old_sessions.clear();
     }
 
+    Status start() override {
+        return Status::OK();
+    }
+
     bool shutdown(Milliseconds timeout) override {
         return true;
     }
 
-    Stats sessionStats() const override {
-        return {};
-    }
+    void appendStats(BSONObjBuilder*) const override {}
 
     size_t numOpenSessions() const override {
         stdx::unique_lock<stdx::mutex> lock(_mutex);
@@ -169,10 +173,12 @@ public:
         return true;
     }
 
-    Stats sessionStats() const override {
-        return {};
+    Status start() override {
+        return Status::OK();
     }
 
+    void appendStats(BSONObjBuilder*) const override {}
+
     size_t numOpenSessions() const override {
         return 0;
     }
