@@ -65,6 +65,7 @@ private:
         MONGO_DISALLOW_COPYING(PeriodicJobImpl);
 
     public:
+        friend class PeriodicRunnerImpl;
         PeriodicJobImpl(PeriodicJob job, ClockSource* source, ServiceContext* svc);
 
         void start();
@@ -72,8 +73,6 @@ private:
         void resume();
         void stop();
 
-        bool isAlive();
-
         enum class ExecutionStatus { NOT_SCHEDULED, RUNNING, PAUSED, CANCELED };
 
     private:
